#!/usr/bin/perl
#
# --
# check_ism - Check Processes running under ISM (Intergraph Interface Service Manager)
#
# Copyright (C) 2016 Florian Fuchs (florian.fuchs@k.roteskreuz.at)
#
# --
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 3
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, 
# MA 02111-1307, USA.
#
#
# The script checks if the processes of I/CAD (Intergraphs Computer Aided Dispatch, sold by 
# Hexagon Safety & Infrastructure) started by ISM (Intergraph Interface Service Manager) 
# currently have the intended state. Depending on the type of process this means a warning 
# or critical state is reached, if the process does not run on any machine, on two machines 
# at the same time or on the backup server. 

use strict;
use warnings;

use Getopt::Long qw(:config no_ignore_case);	# we want the options to be case-sensitive

use constant VERSION => "1.0 (2016-06-14)";		# version number of the script

my %ERRORS=('OK'=>0,'WARNING'=>1,'CRITICAL'=>2,'UNKNOWN'=>3);			# Nagios return values

my $CHECK_NT="/usr/local/nagios/libexec/check_nt -v PROCSTATE -d SHOWALL";	# check_nt command
my $DEFAULT_PORT = "12489";													# check_nt default port
my $DEFAULT_TIMEOUT = "300";												# check_nt default timeout		


#
# usage(optional errormessage)
#
sub usage 
{
	my $message = $_[0];

	# if we have a message, add it to the output
	if (defined $message && length $message) 
	{
		# if there is no \n at the end of the string, add one
		$message .= "\n"
         	   unless $message =~ /\n$/;
   	}

	# get the script name
   	my $command = $0;
   	$command =~ s#^.*/##;

	#get version number
	my $version = VERSION;
	# print error message
   	print STDERR <<EOT;

$command (Version $version)

$message
usage: $command -H1 <host_address1> -H2 <host_address2> -s <processname> [-w {1,2,s,d,n}] [-c {1,2,s,d,n}] [-p <portnumber>] [-t <timeout in seconds>]

    -H1 <host_address1>    hostname or ip address of first host to check
    -H2 <host_address2>    hostname or ip address of second host to check
    -s                     processname to check
    -w                     warn if (optional)
        1                  service runs only on host1
        2                  service runs only on host2
        s                  service runs only on one host but not both
        d                  service runs on both hosts
        n                  service runs neither on host1 nor or host2
        combination also possible, e.g. d,n
    -c                     critical if (optional)
        1                  service runs only on host1
        2                  service runs only on host2
        s                  service runs only on one host but not both
        d                  service runs on both hosts
        n                  service runs neither on host1 nor or host2
        combination also possible, e.g. d,n
    -p                     port number (optional)
    -t                     timeout in seconds (optional)


Examples:

    check_ism.pl -H1 sv001 -H2 sv003 -s listen.exe
    (only print the state)
	
    check_ism.pl -H1 172.27.5.1 -H2 172.27.5.2 -s patidokserver.exe -c n,d -w 1 
    (critical if the service is not running at all or running on both server, 
     warn if running on server 1)

    check_ism.pl -H1 sv001 -H2 sv003 -s listen.exe -w s -c n
    (warn if the service is only running on one server, critical if not running 
     at all)

EOT
	# exit the script
   	NagExit("UNKNOWN");
}


#
# CheckOptions(@array)
#
# Check if the user only provided allowed options (1,2,s,d,n).
#
sub CheckOptions
{
	my @options = @{$_[0]};
	
	foreach (@options)
	{
		if ($_ ne "1" && 
			$_ ne "2" &&
			$_ ne "s" &&
			$_ ne "d" &&
			$_ ne "n")
		{
			usage("Unknown option for critical or warning parameter: $_\n");
		}
	}
}



#
# usage (OK|WARNING|CRITICAL|UNKNOWN, [message]
#
sub NagExit 
{
	my $exitcode = $_[0];
	my $message = $_[1];

        # if we have a message, add it to the output
        if (defined $message && length $message)
        {
                # if there is no \n at the end of the string, add one
                $message .= "\n"
                   unless $message =~ /\n$/;

		if ($exitcode eq "OK")
		{
			$message = "OK: ".$message;
		}
		elsif ($exitcode eq "WARNING")
		{
			$message = "WARNING: ".$message;
		}
		elsif ($exitcode eq "CRITICAL")
		{
			$message = "CRITICAL: ".$message;
		}
		elsif ($exitcode eq "UNKNOWN")
		{
			$message = "UNKNOWN: ".$message;
		}
		print "$message";
      }

	exit $ERRORS{$exitcode};
}

my $host1;		# host name or ip address of the first server
my $host2;		# host name or ip address of the second server
my $service;	# service to be checked
my @critical;	# critical states
my @warning;	# warning states
my $port;		# port
my $timeout;	# timeout
my $execute1;	# check_nt command to be executed for host1
my $execute2;	# check_nt command to be executed for host2
my $result1;	# returned result from host1
my $result2;	# returned result from host2

# get the arguments
GetOptions ("H1=s" => \$host1,    			# mandatory string
			"H2=s" => \$host2,				# mandatory string
			"s=s"  => \$service,			# mandatory string
			"c=s"  => \@critical,			# optional string
			"w=s"  => \@warning,			# optional string
			"p=i"  => \$port,				# optional integer
			"t=i"  => \$timeout)			# optional integer
or usage("Invalid commmand line options.");

# check if we have all mandatory arguments
usage("The host1 (argument -H1) must be specified.")
	unless defined $host1;
   
usage("The host2 (argument -H2) must be specified.")
	unless defined $host2;
	
usage("The service (argument -s) must be specified.")
	unless defined $service;

# separate values using comma as described at http://perldoc.perl.org/Getopt/Long.html
@critical = split(/,/,join(',',@critical));
@warning = split(/,/,join(',',@warning));

# check options, if there are some
CheckOptions(\@critical)
	unless not @critical;
	
CheckOptions(\@warning)
	unless not @warning;
	
# if the user did not define a port, use the default
$port = $DEFAULT_PORT
	unless defined $port;

# if the user did not define a timeout, use the default	
$timeout = $DEFAULT_TIMEOUT
	unless defined $timeout;

# build execution commands (we do not need stdout, therefore >/dev/null)
$execute1 = "$CHECK_NT -p $port -t $timeout -H $host1 -l $service >/dev/null";
$execute2 = "$CHECK_NT -p $port -t $timeout -H $host2 -l $service >/dev/null";

# execute the commands ( if the process is running, the result will be 0)
$result1 = system("$execute1");
$result2 = system("$execute2");

# service is running on both servers
if ($result1 == 0 && $result2 == 0)
{
	# if the service must not run on both servers (option "d"), issue critical
	if (@critical && grep(/^d$/, @critical))
	{
		NagExit("CRITICAL", "$service is running on $host1 and $host2.");
	}
	
	# if the service must not run on both servers (option "d"), issue warning
	if (@warning && grep(/^d$/, @warning))
	{
		NagExit("WARNING", "$service is running on $host1 and $host2.");
	}
}
# service is running only on host1
elsif($result1 == 0 && $result2 != 0)
{
	# if the service must not run on host1 (option "1"), issue critical
	if (@critical && grep(/^1$/, @critical))
	{
		NagExit("CRITICAL", "$service is running on $host1 instead of $host2.");
	}
	
	# if the service must not run on host1 (option "1"), issue warning
	if (@warning && grep(/^1$/, @warning))
	{
		NagExit("WARNING", "$service is running on $host1 instead of $host2.");
	}
	
	# if the service must not run on a single host (option "s"), issue critical
	if (@critical && grep(/^s$/, @critical))
	{
		NagExit("CRITICAL", "$service is running only on $host1 and not on $host2.");
	}
	
	# if the service must not run on a single host (option "s"), issue warning
	if (@warning && grep(/^s$/, @warning))
	{
		NagExit("WARNING", "$service is running only on $host1 and not on $host2.");
	}
}
# service is running only on host2
elsif($result1 != 0 && $result2 == 0)
{
	# if the service must not run on host2 (option "2"), issue critical
	if (@critical && grep(/^2$/, @critical))
	{
		NagExit("CRITICAL", "$service is running on $host2 instead of $host1.");
	}
	
	# if the service must not run on host2 (option "2"), issue warning
	if (@warning && grep(/^2$/, @warning))
	{
		NagExit("WARNING", "$service is running on $host2 instead of $host1.");
	}
	
	# if the service must not run on a single host (option "s"), issue critical
	if (@critical && grep(/^s$/, @critical))
	{
		NagExit("CRITICAL", "$service is running only on $host2 and not on $host1.");
	}
	
	# if the service must not run on a single host (option "s"), issue warning
	if (@warning && grep(/^s$/, @warning))
	{
		NagExit("WARNING", "$service is running only on $host2 and not on $host1.");
	}
}
# service is running nowhere
else
{
	# if the service does not run on any host (option "n"), issue critical
	if (@critical && grep(/^n$/, @critical))
	{
		NagExit("CRITICAL", "$service is neither running on $host1 nor on $host2.");
	}
	
	# if the service does not run on any host (option "n"), issue warning
	if (@warning && grep(/^n$/, @warning))
	{
		NagExit("WARNING", "$service is neither running on $host1 nor on $host2.");
	}
}

#
# if we are here, everything is fine
#

# service is running on both servers
if ($result1 == 0 && $result2 == 0)
{
	NagExit("OK", "$service is running on $host1 and on $host2.");
}
# service is running only on host1
elsif($result1 == 0 && $result2 != 0)
{
	NagExit("OK", "$service is running on $host1 and not on $host2.");
}
# service is running only on host2
elsif($result1 != 0 && $result2 == 0)
{
	NagExit("OK", "$service is running on $host2 and not on $host1.");
}
# service is running nowhere
else
{
	NagExit("OK", "$service is neither running on $host1 nor on $host2.");
}








